// Copyright 2015-2017 Parity Technologies (UK) Ltd.
// This file is part of Parity.

// This software is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Parity.  If not, see <http://www.gnu.org/licenses/>.

use cita_types::H256;
use state::backend::*;
use util::{DBTransaction, HashDB, JournalDB, UtilError};

pub struct StateDB {
    /// Backing database.
    db: Box<JournalDB>,
}

impl StateDB {
    pub fn new(db: Box<JournalDB>) -> StateDB {
        StateDB { db }
    }

    /// Clone the database.
    pub fn boxed_clone(&self) -> StateDB {
        StateDB {
            db: self.db.boxed_clone(),
        }
    }

    /// Journal all recent operations under the given era and ID.
    pub fn journal_under(
        &mut self,
        batch: &mut DBTransaction,
        now: u64,
        id: &H256,
    ) -> Result<u32, UtilError> {
        self.db.journal_under(batch, now, id)
    }
    pub fn mark_canonical(
        &mut self,
        batch: &mut DBTransaction,
        now: u64,
        id: &H256,
    ) -> Result<u32, UtilError> {
        self.db.mark_canonical(batch, now, id)
    }

    /// Returns underlying `JournalDB`.
    pub fn journal_db(&self) -> &JournalDB {
        &*self.db
    }
}

impl Backend for StateDB {
    fn as_hashdb(&self) -> &HashDB {
        self.db.as_hashdb()
    }

    fn as_hashdb_mut(&mut self) -> &mut HashDB {
        self.db.as_hashdb_mut()
    }
}
